#include "spdlog.hpp"
#include "config.hpp"

namespace myx {

namespace log {

Logger default_logger( "default" );


Logger::Logger( std::string name ) :
	m_logger( nullptr ),
	m_name( std::move( name ) ),
	m_outputPattern( "[%H:%M:%S %z] [%n] [%^---%L---%$] %v" ),
	m_baseFileName( fmt::format( "{}_{}", PROJECT_NAME, "st" ) ),
	m_maxRotatingFileSize( rotatingFileSize ),
	m_maxRotatingFilesCount( 3 ),
	m_outputLevel( spdlog::level::trace ),
	m_flushLevel( spdlog::level::warn ),
	m_nullSink( nullptr ),
	m_stdoutSink( nullptr ),
	m_stderrSink( nullptr ),
	m_syslogSink( nullptr ),
	m_basicFileSink( nullptr ),
	m_rotatingFileSink( nullptr ),
	m_dailyFileSink( nullptr )
{
	m_logger = std::make_shared< spdlog::logger >( m_name, nullptr );
	m_logger->sinks().clear();
	m_logger->flush_on( m_flushLevel );
	m_logger->set_level( m_outputLevel );
	m_logger->set_pattern( m_outputPattern );
}


spdlog::level::level_enum Logger::outputLevel() const
{
	return( m_outputLevel );
}


void Logger::setOutputLevel( const spdlog::level::level_enum outputLevel )
{
	m_outputLevel = outputLevel;
}


spdlog::level::level_enum Logger::flushLevel() const
{
	return( m_flushLevel );
}


void Logger::setFlushLevel( const spdlog::level::level_enum flushLevel )
{
	m_flushLevel = flushLevel;
}


std::string Logger::outputPattern() const
{
	return( m_outputPattern );
}


void Logger::setOutputPattern( const std::string& outputPattern )
{
	m_outputPattern = outputPattern;
}


std::shared_ptr< spdlog::logger > Logger::logger()
{
	return( m_logger );
}


std::string Logger::name() const
{
	return( m_name );
}


void Logger::setName( const std::string& name )
{
	m_name = name;
}


std::string Logger::baseFileName() const
{
	return( m_baseFileName );
}


void Logger::setBaseFileName( const std::string& baseFileName )
{
	m_baseFileName = baseFileName;
}


void Logger::enableNullSink()
{
	if ( m_nullSink == nullptr )
	{
		m_nullSink = std::make_shared< spdlog::sinks::null_sink_st >();
		m_logger->sinks().push_back( m_nullSink );
	}
}


void Logger::disableNullSink()
{
	if ( m_nullSink != nullptr )
	{
		auto sinks = m_logger->sinks();
		sinks.erase( std::remove( sinks.begin(), sinks.end(), m_nullSink ), sinks.end() );
		m_nullSink.reset();
		m_nullSink = nullptr;
	}
}


void Logger::enableStdoutSink()
{
	if ( m_stdoutSink == nullptr )
	{
		m_stdoutSink = std::make_shared< spdlog::sinks::ansicolor_stdout_sink_st >();
		m_logger->sinks().push_back( m_stdoutSink );
	}
}


void Logger::disableStdoutSink()
{
	if ( m_stdoutSink != nullptr )
	{
		auto sinks = m_logger->sinks();
		sinks.erase( std::remove( sinks.begin(), sinks.end(), m_stdoutSink ), sinks.end() );
		m_stdoutSink.reset();
		m_stdoutSink = nullptr;
	}
}


void Logger::enableStderrSink()
{
	if ( m_stderrSink == nullptr )
	{
		m_stderrSink = std::make_shared< spdlog::sinks::ansicolor_stderr_sink_st >();
		m_logger->sinks().push_back( m_stderrSink );
	}
}


void Logger::disableStderrSink()
{
	if ( m_stderrSink != nullptr )
	{
		auto sinks = m_logger->sinks();
		sinks.erase( std::remove( sinks.begin(), sinks.end(), m_stderrSink ), sinks.end() );
		m_stderrSink.reset();
		m_stderrSink = nullptr;
	}
}


void Logger::enableSyslogSink()
{
	if ( m_syslogSink == nullptr )
	{
		m_syslogSink = std::make_shared< spdlog::sinks::syslog_sink_st >();
		m_logger->sinks().push_back( m_syslogSink );
	}
}


void Logger::disableSyslogSink()
{
	if ( m_syslogSink != nullptr )
	{
		auto sinks = m_logger->sinks();
		sinks.erase( std::remove( sinks.begin(), sinks.end(), m_syslogSink ), sinks.end() );
		m_syslogSink.reset();
		m_syslogSink = nullptr;
	}
}


void Logger::enableBasicFileSink()
{
	if ( m_basicFileSink == nullptr )
	{
		m_basicFileSink = std::make_shared< spdlog::sinks::basic_file_sink_st >( m_baseFileName );
		m_logger->sinks().push_back( m_basicFileSink );
	}
}


void Logger::disableBasicFileSink()
{
	if ( m_basicFileSink != nullptr )
	{
		auto sinks = m_logger->sinks();
		sinks.erase( std::remove( sinks.begin(), sinks.end(), m_basicFileSink ), sinks.end() );
		m_basicFileSink.reset();
		m_basicFileSink = nullptr;
	}
}


void Logger::enableRotatingFileSink()
{
	if ( m_rotatingFileSink == nullptr )
	{
		m_rotatingFileSink = std::make_shared< spdlog::sinks::rotating_file_sink_st >( m_baseFileName, m_maxRotatingFileSize, m_maxRotatingFilesCount );
		m_logger->sinks().push_back( m_rotatingFileSink );
	}
}


void Logger::disableRotatingFileSink()
{
	if ( m_rotatingFileSink != nullptr )
	{
		auto sinks = m_logger->sinks();
		sinks.erase( std::remove( sinks.begin(), sinks.end(), m_rotatingFileSink ), sinks.end() );
		m_rotatingFileSink.reset();
		m_rotatingFileSink = nullptr;
	}
}


void Logger::enableDailyFileSink()
{
	if ( m_dailyFileSink == nullptr )
	{
		const int rotationHour = 23;
		const int rotationMinute = 59;
		m_dailyFileSink = std::make_shared< spdlog::sinks::daily_file_sink_st >( m_baseFileName, rotationHour, rotationMinute );
		m_logger->sinks().push_back( m_dailyFileSink );
	}
}


void Logger::disableDailyFileSink()
{
	if ( m_dailyFileSink != nullptr )
	{
		auto sinks = m_logger->sinks();
		sinks.erase( std::remove( sinks.begin(), sinks.end(), m_dailyFileSink ), sinks.end() );
		m_dailyFileSink.reset();
		m_dailyFileSink = nullptr;
	}
}

} // namespace log

} // namespace myx
